/*
 * Copyright 2014 OGIS-RI All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.co.ogis_ri.optaplanner.jobSolution.domain;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.optaplanner.core.api.domain.solution.PlanningEntityCollectionProperty;
import org.optaplanner.core.api.domain.solution.PlanningSolution;
import org.optaplanner.core.api.domain.solution.Solution;
import org.optaplanner.core.api.domain.valuerange.ValueRangeProvider;
import org.optaplanner.core.api.score.buildin.hardsoftbigdecimal.HardSoftBigDecimalScore;

@PlanningSolution
public class JobSolution extends AbstractPersistable implements
		Solution<HardSoftBigDecimalScore> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private List<People> peopleList;

	private List<JobAssignment> jobAssignmentList;

	private List<Job> jobList;

	private HardSoftBigDecimalScore score;

	private List<Day> dayList;

	private SolutionParameter solutionParameter;

	public JobSolution() {
	}

	public JobSolution(final int id, final String name,
			final List<People> peopleList, final List<Job> jobList,
			final List<JobAssignment> jobAssignmentList, final List<Day> dayList) {
		super(id, name);
		this.peopleList = peopleList;
		this.jobList = jobList;
		this.jobAssignmentList = jobAssignmentList;
		this.dayList = dayList;

		// パラメータを求める
		this.solutionParameter = new SolutionParameter(9999,
				"SolutionParameter");

		// 平均的な回数を求める
		this.solutionParameter.setCountAverage((float) jobAssignmentList.size()
				/ (float) peopleList.size());

		// 最大と最小の日の値を求める
		int maxDay = Integer.MIN_VALUE;
		int minDay = Integer.MAX_VALUE;

		for (JobAssignment jobAssignment : jobAssignmentList) {
			if (jobAssignment.getJob().getDay().getDayValue() > maxDay) {
				maxDay = jobAssignment.getJob().getDay().getDayValue();
			}

			if (jobAssignment.getJob().getDay().getDayValue() < minDay) {
				minDay = jobAssignment.getJob().getDay().getDayValue();
			}
		}

		this.solutionParameter.setMaxDay(maxDay);
		this.solutionParameter.setMinDay(minDay);

		// 平均的な日の間隔を求める
		int dayIntervalSum = 0;
		int count = 0;

		for (int i = 0; i < jobAssignmentList.size() - peopleList.size(); i++) {

			dayIntervalSum = dayIntervalSum
					+ jobAssignmentList.get(i + peopleList.size()).getJob()
							.getDay().getDayValue()
					- jobAssignmentList.get(i).getJob().getDay().getDayValue();
			count++;
		}

		this.solutionParameter.setIntervalDayAverage((float) dayIntervalSum
				/ (float) count);	
	}

	@ValueRangeProvider(id = "peopleList")
	public List<People> getPeopleList() {
		return this.peopleList;
	}

	public void setPeopleList(final List<People> peopleList) {
		this.peopleList = peopleList;
	}

	@PlanningEntityCollectionProperty()
	public List<JobAssignment> getJobAssignmentList() {
		return this.jobAssignmentList;
	}

	public void setJobAssignmentList(final List<JobAssignment> jobAssignmentList) {
		this.jobAssignmentList = jobAssignmentList;
	}

	@Override
	public HardSoftBigDecimalScore getScore() {
		return this.score;
	}

	@Override
	public void setScore(final HardSoftBigDecimalScore score) {
		this.score = score;
	}

	@Override
	public Collection<? extends Object> getProblemFacts() {
		List<Object> facts = new ArrayList<Object>();
		facts.addAll(this.peopleList);
		facts.addAll(this.jobList);
		facts.addAll(this.dayList);
		facts.add(this.solutionParameter);
		return facts;
	}

}
